;*************************************************************************
; DICEGAME v.9 beta
; for USGARD
;
; by Doug Melton
;
; started     : 5/28/97
; last updated: 6/15/97
;
;*************************************************************************

#include "usgard.h"

#include "displib.h"
#include "usgdlib.h"
#include "winlib.h"

.org 0
.db "Dice Game v.9",0
.db DISPLIB, USGDLIB, WINLIB, 0

;*************************************************************************
;;;Game Variables
randNum         = TEXT_MEM      ; Byte

asciiNumb       = TEXT_MEM+1    ; 6 Bytes       } 
tmp             = TEXT_MEM+1    ; Word          } temporary storage
die             = TEXT_MEM+4    ; byte          }

score           = TEXT_MEM+7    ; byte

arrow           = TEXT_MEM+8    ; byte number under which arrow is
arrowX          = TEXT_MEM+9    ; byte position of arrow (x-coord)

roll            = TEXT_MEM+10   ; byte current dice roll

pieces          = TEXT_MEM+11   ; 12 bytes, the individual pieces

;*************************************************************************
;main program

top:
        ld      a, 4
        out     (5), a

        xor     a
        LIB_CALL_(DISPLIB, D_SET)

        inc     a
        ld      (arrow), a

        ld      a, 78
        ld      (score), a

;;;show title screen
titleScrn:
        call    CLEARLCD

        ld      bc, $392F
        ld      (CURSOR_X), bc
        ld      hl, (PROGRAM_ADDR)      
        call    D_ZM_STR                

        ld      bc, $0107               
        ld      hl, $0E28               
        ld      ix, &strTitle
        ld      de, (PROGRAM_ADDR)      
        ld      a, %111                  
        LIB_CALL_(WINLIB,WIN_PSHOW)

        call    OTH_PAUSE

startOfGame:
        ld      bc, $0004
        ld      hl, $1010               
        ld      ix, &strNumbers
        ld      a, %110                  
        LIB_CALL_(WINLIB,WIN_PSHOW)             ;display 1-12

        ld      bc, $0519
        ld      hl, $0614
        xor     a                  
        LIB_CALL_(WINLIB,WIN_PSHOW)             ;window for dice

displayScore:
        ld      a, (roll)
        or      a                               ;if they have 0 as a dice roll...
        call    z, &rollDice                    ;roll dice

        ld      bc, $3971
        ld      (CURSOR_X), bc
        ld      a, (score)
        ld      b, a
        ld      a, (roll)
        add     a, b
        LIB_CALL_(DISPLIB,D_A_DEC)              ;display their score

        ld      a, (arrow)
        sla     a
        ld      c, a
        sla     a
        sla     a
        add     a, c
        sub     4
        ld      (arrowX), a
        ld      c, a

        ld      b, $0C
        ld      (CURSOR_X), bc
        ld      hl, &strArrow
        call    D_ZM_STR                        ;display arrow

        ld      a, (roll)                       ;check for game over
        ld      b, a
        ld      hl, pieces
gameOverLoop:
        ld      a, (hl)
        or      a
        jr      z, noGameOver
        inc     hl
        djnz    gameOverLoop

gameOver:
        ld      a, (IY+5)
        xor     %00001000
        ld      (IY+5), a

        ld      bc, $0603
        ld      (CURSOR_ROW), bc
        ld      hl, &strGameOver
        call    D_ZT_STR                        ;display "GAME OVER"

        ld      b, 30
gameOverWait:
        halt
        djnz    gameOverWait

        call    GET_KEY
        or      a
        jr      z, gameOver

;;;;
;        call    &LowScoreHand
;-----------------------------------------------------------------------------
;Handle Low-score
;-----------------------------------------------------------------------------
LowScoreHand:
        ld      hl, &LowScore
        ld      a, (roll)
        ld      b, a
        ld      a, (score)                      
        add     a, b                            ;(hl) contains Lowscore, a actual score
        
        cp      (hl)
        jr      nc, quit                        ;they don't have a Low score

        ld      (hl), a                         ;save new low score
        ld      hl, USG_BITS
        set     0, (hl)                         ;make zshell recalc. checksum

        ld      bc, $030A               
        ld      hl, $0A20               
        ld      ix, &strNewLowScore
        ld      de, (PROGRAM_ADDR)      
        ld      a, %111
        LIB_CALL_(WINLIB,WIN_PSHOW)

        ld      hl, $0904
        ld      (CURSOR_ROW), hl
        ld      ix, &LowScrName
        ld      c, 3
        ld      a, 00110000b
        LIB_CALL_(USGDLIB,INPST)               ;input string?

;        ret
;;;;
quit:
        call    GET_KEY                         ;take last key out of buffer
        ret                                     ;quit game

noGameOver:
keyLoop:
        halt
        call    GET_KEY
        cp      K_EXIT
        jr      z, quit
        cp      K_LEFT
        jr      z, left
        cp      K_RIGHT
        jr      z, right
        cp      K_SECOND
        jr      z, enter
        jr      keyLoop

left:
        ld      a, (arrow)
        dec     a
        or      a
        jr      nz, rightLeft
        ld      a, 12
        jr      rightLeft

right:
        ld      a, (arrow)
        inc     a
        cp      13
        jr      nz, rightLeft
        ld      a, 1

rightLeft:
        ld      (arrow), a

        ld      hl, VIDEO_MEM+(13*16)
        ld      bc, (16*5)-1
        call    OTH_CLEAR                       ;erase arrow

        jp      &displayScore

enter:
        ld      bc, (arrow)                     ;c is the selected number
        ld      b, 0
        ld      hl, pieces-1
        add     hl, bc
        ld      a, (hl)
        or      a
        jr      nz, enter2                      ;they've already taken that piece

        ld      a, (roll)
        cp      c
        jr      c, enter2                       ;their roll isn't high enough
        sub     c
        ld      (roll), a

        dec     (hl)

        ld      a, (score)
        sub     c
        ld      (score), a

        ld      b, $05
        ld      a, (arrowX)
        sub     2
        ld      c, a
        ld      (CURSOR_X), bc
        ld      hl, &strBlanks
        call    D_ZM_STR                        ;erase number

enter2:
        jp      &displayScore


;********************* the dice rolling routine **************************
;Description:   rolls some dice
;Parameters:    b, number of dice (set in routine)
;               hl, mem where to put sprites (set in routine)
;Destroys:      af, bc, de, hl

rollDice:
        ld      hl, VIDEO_MEM+$01D5
        ld      bc, $0200
rollDice2:
;        push    bc
;        push    hl
;        call    RANDOM
;        pop     hl
;        pop     bc
;;;asdf
;        push    af
;        push    bc
;        push    hl
;        ld      bc, $3964
;        ld      (CURSOR_X), bc
;        LIB_CALL_(DISPLIB,D_A_DEC)              ;display their score
;        pop     hl
;        pop     bc
;        pop     af
;;;
;Chris Busch's Random Routine:
        ld      a, (randNum)
        ld      d, a
        sla     d
        sla     d
        add     a, d
        sla     d
        sla     d
        add     a, d
        ld      d, a
        ld      a, r
        add     a, d
        ld      (randNum), a
;;;end of the random routine
        and     7
        inc     a
        cp      7
        jr      nc, rollDice2

        call    &DispDie
        inc     hl
        inc     hl
        inc     hl

        add     a, c
        ld      c, a
        djnz    rollDice2

        ld      (roll), a

        ret

;***************  Display Die Routine  ***********************************
;Description:   Display die defined by register a
;Parameters:    hl - address of upper-left of die (die is 3 bytes wide)
;               a - number on die
;Destroys:      de

DispDie:            
        push    af
        push    bc
        push    hl
        ld      (die), a
        ld      c, a

        ld      a, $01
        ld      (tmp), a
        ld      a, $80
        ld      (tmp+2), a
        ld      a, $FF
        call    &DispDieLoad3
        ld      a, $02
        ld      (tmp), a
        ld      a, $40
        ld      (tmp+2), a
        xor     a
        call    &DispDieLoad3

        ld      b, 3
DispDieLoop:
        ld      (tmp+1), a
        push    hl
        ld      hl, tmp+1
        ld      a, b
        cp      1
        jr      z, DDrow3
        cp      2
        jr      z, DDrow2
DDrow1:
        ld      a, c
        cp      1
        jr      z, DDLoopDraw
        set     6, (hl)
        set     7, (hl)
        cp      4
        jr      c, DDLoopDraw
        set     0, (hl)
        set     1, (hl)
        cp      6
        jr      nz, DDLoopDraw
        set     3, (hl)
        set     4, (hl)
        jr      DDLoopDraw
DDrow2:
        ld      a, c
        bit     0, a
        jr      z, DDLoopDraw
        set     3, (hl)
        set     4, (hl)
        jr      DDLoopDraw
DDrow3:
        ld      a, c
        cp      1
        jr      z, DDLoopDraw
        set     0, (hl)
        set     1, (hl)
        cp      4
        jr      c, DDLoopDraw
        set     6, (hl)
        set     7, (hl)
        cp      6
        jr      nz, DDLoopDraw
        set     3, (hl)
        set     4, (hl)
DDLoopDraw:
        ld      a, (tmp+1)
        pop     hl
        call    &DispDieLoad3
        call    &DispDieLoad3
        xor     a
        call    &DispDieLoad3
        djnz    DispDieLoop

        ld      a, $01
        ld      (tmp), a
        ld      a, $80
        ld      (tmp+2), a
        ld      a, $FF
        call    &DispDieLoad3

        pop     hl
        pop     bc
        pop     af
        ret

DispDieLoad3:
        ld      (tmp+1), a
        push    bc
        ex      de, hl
        ld      hl, tmp
        ld      bc, 3
        ldir
        ex      de, hl
        ld      de, 13
        add     hl, de
        pop     bc
        ret

;*************************************************************************
Strings:
strTitle:       .db LF,XTAB,30,"by Doug Melton",LF
                .db XTAB,6,"<joeblow969@hotmail.com>",LF,LF
strLowScore:    .db XTAB,17,"Low Score:",BNUM

LowScore:       .db 78                          ;This is standard Low-score
LowScrName0:    .db XTAB,4,"by "
LowScrName:     .db "DUG",0

strGameOver:    .db "GAME OVER",0

strNewLowScore: .db XTAB,13,"New Low Score!",LF
                .db XTAB,9,"Enter Your Initials:",0

strNumbers:     .db XTAB,5,"1",XTAB,6,"2",XTAB,6
                .db "3",XTAB,6,"4",XTAB,6
                .db "5",XTAB,6,"6",XTAB,6
                .db "7",XTAB,6,"8",XTAB,6
                .db "9",XTAB,4,"10  11  12",0

strArrow:       .db $1E                 ;maybe delete the "0" here???
strBlanks:      .db "       ",0

.end

